/**
*	WARNING: Do Not Remove This Section
*
*	$RCSfile: uniformization_qureshi_sanders.h,v $
*	$Revision: 1.8 $
*	$Date: 2006/02/13 14:47:31 $
*	$Author: zapreevis $
*	
*	MRMC is a model checker for discrete-time and continuous-time Markov reward models.
*	It supports reward extensions of PCTL and CSL (PRCTL and CSRL), and allows for the
*	automated verification of properties concerning long-run and instantaneous rewards
*	as well as cumulative rewards.
*	
*	Copyright (C) The University of Twente, 2004-2006.
*	Authors: Maneesh Khattri
*
*	This program is free software; you can redistribute it and/or
*	modify it under the terms of the GNU General Public License
*	as published by the Free Software Foundation; either version 2
*	of the License, or (at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program; if not, write to the Free Software
*	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*
*	Our contacts:
*		Formal Methods and Tools Group, University of Twente,
*		P.O. Box 217, 7500 AE Enschede, The Netherlands,
*		Phone: +31 53 4893767, Fax: +31 53 4893247,
*		E-mail: mrmc@cs.utwente.nl
*
*	Source description: Uniformization based algorithm by Qureshi&Sanders for CSRL - U.
*	Uses: DEF: bitset.h, sparse.h, label.h, runtime.h
*		LIB: bitset.c, sparse.c, label.c, runtime.c
*	Remarks: Uniformization based algorithm by Qureshi&Sanders for CSRL - U.
*		For discussion see:
*		1. M.A. Qureshi and W.H.Sanders. Reward Model Solution Methods with Impulse and Rate Rewards:
*		An Algorithm and Numerical Results. Performance Evaluation, 20(4), 1994.
*		2. M.A. Qureshi and W.H. Sanders. A new methodology for calculating distributions of reward
*		accumulated during a finite interval. Fault-Tolerant Computing Symposium, IEEE CS Press,
*		pp. 116-125, 1996.
*			  
*		Only valid for MRMs without impulse rewards.
*/

/*****************************************************************************
name		: uniformization_qureshi_sanders
role       	: solve t-r-bounded until formula in MRM without impulse rewards
		  using uniformization.
@param		: int state: the starting state.
@param		: bitset *phi: satisfaction relation for phi formula.
@param		: bitset *psi: satisfaction relation for psi formula.
@param		: double supi: sup I.
@param		: double supj: sup J.
@param		: int: print.
@return		: double *: result of the until formula for all states.
remark 	        :
******************************************************************************/
extern double * uniformization_qureshi_sanders(bitset *, bitset *, double, double, int);

extern double omega(int *, double, int, int, double *);

